/*
 * Internet checksum routines, optimized for the Rabbit 2000/3000
 *
 * (C) 2002 Compendium Technologies, Inc.
 * (C) 2002 Softools, Inc.
 * All rights reserved.
 *
 * This software is the proprietary information of Compendium Technologies, Inc.
 * and Softools, Inc. Use is subject to license terms.
 */

#include "lwip/debug.h"
#include "lwip/arch.h"
#include "lwip/def.h"
#include "lwip/inet.h"

/**
 * chksum -- Rabbit 2000/3000 optimized assembly routines for computing the
 * basis of the internet checksum.
 */
#asm
_chksum::
    ex      de,hl       ; DE = dataptr
    ld      hl,(sp+4)   ; HL = len
    ld      a,l
    or      a,h         ; Test for 0 and exit with 0 if 0
    ret     z           ; Exit - HL=0

    rr      hl          ; len / 2
    ex      af,af'      ; CF'=odd byte flag
    dec     hl          ; Inc H if L != 0
    inc     l
    inc     h
    ld      b,l
    ld      c,h
    bool    hl          ; Check for word count == 0 (CF=0)
    ld      l,h         ; Check sum in HL = 0
    jr      z,..one     ; Jump if only 1 byte

    ex      de,hl       ; HL = ptr
..loop:                 ; sum words
    ld      a,e         ; Load word
    adc     a,(hl)
    ld      e,a
    inc     hl
    ld      a,d
    adc     a,(hl)
    ld      d,a
    inc     hl
    djnz    ..loop
    dec     c
    jr      nz,..loop

    ex      de,hl       ; HL = Checksum
    adc     hl,bc       ; Add 0 twice to pick up carries
    adc     hl,bc

    ex      af,af'      ; Get odd byte flag
    ret     nc

..one:
    ld      a,(de)      ; Last byte
    ld      c,a         ; B=0
    add     hl,bc       ; Add byte
    ld      c,b         ; BC=0
    adc     hl,bc       ; Add 0 twice to pick up carries
    adc     hl,bc
    ret
#endasm

/**
 * chksum_pbuf -- worker routine that computes the checksum on a pbuf chain.
 */
#asm
_chksum_pbuf::
    ld      bc,0         ; acc
    ld      e,0          ; e = swapped

..loop:
    ld      iy,hl        ; p
    bool    hl           ; p != NULL
    jr      z,..done
    exx                  ; save registers
    ;; Replaced with inline version
    ;;ld      hl,(iy+8)    ; len is offset +8 in pbuf
    ;;push    hl
    ;;ld      hl,(iy+4)    ; payload is offset +4 in pbuf
    ;;push    hl
    ;;call    _chksum      ; Call chksum for this buffer
    ;;add     sp,4
    ;; Start inline replacement of _chksum
    ld      hl,(iy+4)   ; payload 
    ex      de,hl       ; DE = payload
    ld      hl,(iy+8)   ; HL = len
    ld      a,l
    or      a,h         ; Test for 0 and exit with 0 if 0
    jr      z,..donepbuf ; Exit - HL=0
   

    rr      hl          ; len / 2
    ex      af,af'      ; CF'=odd byte flag
    dec     hl          ; Inc H if L != 0
    inc     l
    inc     h
    ld      b,l
    ld      c,h
    bool    hl          ; Check for word count == 0 (CF=0)
    ld      l,h         ; Check sum in HL = 0
    jr      z,..one     ; Jump if only 1 byte

    ex      de,hl       ; HL = ptr
..inloop:               ; sum words
    ld      a,e         ; Load word
    adc     a,(hl)
    ld      e,a
    inc     hl
    ld      a,d
    adc     a,(hl)
    ld      d,a
    inc     hl
    djnz    ..inloop
    dec     c
    jr      nz,..inloop

    ex      de,hl       ; HL = Checksum
    adc     hl,bc       ; Add 0 twice to pick up carries
    adc     hl,bc

    ex      af,af'      ; Get odd byte flag
    jr      nc,..donepbuf

..one:
    ld      a,(de)      ; Last byte
    ld      c,a         ; B=0
    add     hl,bc       ; Add byte
    ld      c,b         ; BC=0
    adc     hl,bc       ; Add 0 twice to pick up carries
    adc     hl,bc

..donepbuf:
    ;; Done with inline replacement
    ex      de,hl
    ld      hl',de
    exx                  ; Restore registers

    add     hl,bc        ; acc += chksum
    ld      bc,0

    adc     hl,bc        ; Catch carries
    adc     hl,bc        ; Second and final one
   
    ld      bc,hl        ; acc
    ld      hl,(iy+8)    ; len is offset +8 in pbuf
    bit     0,l          ; len % 2 == 0, skip
    jr      z,..no       
    ld      hl,(iy)      ; next is at offset 0 in pbuf
    bool    hl
    jr      z,..no       ; pbuf->next == NULL

    ld      a,e
    xor     a,1          ; Toggle lsb
    ld      e,a
    ld      a,c
    ld      c,b
    ld      b,a          ; Swap bytes in acc
..no:
    ld      hl,(iy)      ; next is at offset 0 in pbuf
    jr      ..loop       ; Loop will test and exit if NULL

..done:
    bit     0,e          ; Check swapped to see if we need to swap the bytes
    jr      z,..exit

    ld      h,c          ; swap bytes acc into hl
    ld      l,b
    ret
    ;;ld      b,a          ; Swap bytes in acc

..exit:
    ld      hl,bc        ; Return acc 
    ret
#endasm

#ifdef NOT_USED
near u16_t
chksum_pbuf(struct pbuf *p)
{
  // This is the original (but optimized) C code
  u32_t acc;
  u8_t swapped;
  
  acc = 0;
  swapped = 0;
  for(; p != NULL; p = p->next) {
    u16_t len = p->len;
    acc += chksum(p->payload, len);
    while(acc >> 16) {
      acc = (acc & 0xffff) + (acc >> 16);
    }    
    if(len % 2 != 0 && p->next != NULL) {
      swapped = 1 - swapped;
      acc = ((acc & 0xff) << 8) | ((acc & 0xff00) >> 8);
    }
  }
 
  if(swapped) {
    acc = ((acc & 0xff) << 8) | ((acc & 0xff00) >> 8);
  }
  return acc;
  //return ~(acc & 0xffff);
}
#endif

/**
 * inet_chksum_pseudo -- calculates the pseudo internet checksum used by
 * TCP and UDP for a pbuf chain.
 */
u16_t
inet_chksum_pseudo(struct pbuf *p,
		   struct ip_addr *src, struct ip_addr *dest,
		   u8_t proto, u16_t proto_len)
{
#asm
    push    hl           ; hl should contain p at this point
    call    _chksum_pbuf ; get summation of the bytes
    add     sp,2         ; looks like this is necessary to recover from push
    ld      bc,hl        ; acc

    ; Now get and sum ip addresses
    ld      hl,(ix+7)    ; src ip address
    ld      iy,hl
    ld      hl,(iy)
    add     hl,bc
    ld      bc,hl
    ld      hl,(iy+2)
    adc     hl,bc
    ld      bc,hl

    ld      hl,(ix+9)    ; dest ip address
    ld      iy,hl
    ld      hl,(iy)
    adc     hl,bc
    ld      bc,hl
    ld      hl,(iy+2)
    adc     hl,bc
    ld      bc,hl

    ld      a,(ix+11)    ; proto
    ld      h,a          ; swap bytes on the way
    ld      l,0
    adc     hl,bc
    ld      bc,hl
    
    ld      hl,(ix+13)   ; proto_len
    ld      a,l          ; swap the bytes
    ld      l,h
    ld      h,a 
    adc     hl,bc
    
    ; At the end, add carries and return value in hl
    ld      bc,0
    adc     hl,bc        ; Catch carries
    adc     hl,bc        ; Second and final one

    ; Need to xor what's in hl
    ld      a,h
    cpl     a
    ld      h,a
    ld      a,l
    cpl     a
    ld      l,a
#endasm
}

#ifdef NOT_USED
u16_t
inet_chksum_pseudo(struct pbuf *p,
		   struct ip_addr *src, struct ip_addr *dest,
		   u8_t proto, u16_t proto_len)
{
  // Original (optimized) C code
  u32_t acc = chksum_pbuf(p);

  acc += (src->addr & 0xffff) + ((src->addr >> 16) & 0xffff) +
         (dest->addr & 0xffff) + ((dest->addr >> 16) & 0xffff) +
         (htons((u16_t)proto) + htons(proto_len));  
  
  while(acc >> 16) {
    acc = (acc & 0xffff) + (acc >> 16);
  }    
  //return ~(acc & 0xffff);
  return ~((u16_t)acc);
}
#endif
